#!/bin/bash
# This script generates SD card disk images suitable for use with QEMU.
#
# Copyright (C) 2011 Ash Charles
# Based on:
#   Narcissus - Online image builder for the angstrom distribution
#   Copyright (C) 2008 - 2011 Koen Kooi
#   Copyright (C) 2010        Denys Dmytriyenko
# and
#   Linaro Images Tools.
#   Author: Guilherme Salgado <guilherme.salgado@linaro.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

LC_ALL=C
set -e

function usage()
{
    echo "This utility generates SD card images suitable for use with QEMU."
    echo "Usage:"
    echo "  $0 <output name> <size in bytes>  <boot>  <rootfs>"
    echo "Example:"
    echo " $0 sd.img 1030225920 ./boot ./rootfs"
}

function check_args()
{
    if [ `whoami` != root ]; then
        echo "this utility must be run from root or using sudo!"
        exit 1
    fi

    echo $#
    if [ $# -ne 4 ]; then
        usage
        exit 1
    fi
    OUTFILE=$1
    SIZE=$2
    BOOT_DIR=$3        
    ROOTFS_DIR=$4
    
    if ! [[ -d ${BOOT_DIR} ]]; then
        echo "boot dir not found at ${BOOT_DIR}! Quitting..."
        exit 1
    fi

    if ! [[ -d ${ROOTFS_DIR} ]]; then
        echo "Rootfs not found at ${ROOTFS_DIR}! Quitting..."
        exit 1
    fi
}

#SIZE=15032385536
#SIZE=1030225920

function make_image()
{
    qemu-img create -f raw ${OUTFILE} ${SIZE}
    
    CYLINDERS=`echo ${SIZE}/255/63/512 | bc`
    {
    echo 128,130944,0x0C,*
    echo 131072,,,-
    } | sfdisk --force -D -uS -H 255 -S 63 -C ${CYLINDERS} ${OUTFILE}  &> /dev/null
    
    # Reverse-engineer the partition setup
    BYTES_PER_SECTOR="$(/sbin/fdisk -l -u ${OUTFILE} | grep Units | awk '{print $9}')"
    VFAT_SECTOR_OFFSET="$(/sbin/fdisk -l -u ${OUTFILE} | grep img1 | awk '{print $3}')"
    EXT3_SECTOR_OFFSET="$(/sbin/fdisk -l -u ${OUTFILE} | grep img2 | awk '{print $2}')"
}



function populate_image() 
{
    LOOP_DEV="/dev/loop1"
    LOOP_DEV_FS="/dev/loop2"
    MNT_DIR=/tmp/tmp_tmp
    echo "[ Format vfat partition ]"
    /sbin/losetup -v -o $(expr ${BYTES_PER_SECTOR} "*" ${VFAT_SECTOR_OFFSET}) ${LOOP_DEV} ${OUTFILE}
    mkfs.vfat -F 32 -n "boot" ${LOOP_DEV}
    
    echo "[ Format ext3 partition ]"
    /sbin/losetup -v -o $(expr ${BYTES_PER_SECTOR} "*" ${EXT3_SECTOR_OFFSET}) ${LOOP_DEV_FS} ${OUTFILE}
    /sbin/mkfs.ext3 -L rootfs ${LOOP_DEV_FS}
    
    echo "[ Copying files to vfat ]"
    mkdir -p ${MNT_DIR}    

    mount ${LOOP_DEV} ${MNT_DIR}

    cp -vr ${BOOT_DIR}/* ${MNT_DIR}

    umount ${LOOP_DEV}
    
    echo "[ Copying file system ]"
    mount ${LOOP_DEV_FS} ${MNT_DIR}
 #   tar xaf ${ROOTFS} -C /mnt
    cp -vr ${ROOTFS_DIR}/*  ${MNT_DIR}
    umount ${LOOP_DEV_FS}
    
    echo "[ Clean up ]"
    echo "rm -fr ${MNT_DIR}"
    rm -fr ${MNT_DIR}
    echo "/sbin/losetup -d ${LOOP_DEV}"
    /sbin/losetup -d ${LOOP_DEV}
    echo "/sbin/losetup -d ${LOOP_DEV_FS}"
    /sbin/losetup -d ${LOOP_DEV_FS}
}

ARGS=$*
check_args $ARGS
make_image
populate_image

